@@ -24,16 +24,24 @@ from meilisearch._utils import iso_to_date_time
 from meilisearch.config import Config
 from meilisearch.errors import version_error_hint_message
 from meilisearch.models.document import Document, DocumentsResults
-from meilisearch.models.index import (
+from meilisearch.models.embedders import (
+    CompositeEmbedder,
     Embedders,
-    Faceting,
+    EmbedderType,
     HuggingFaceEmbedder,
-    IndexStats,
+    OllamaEmbedder,
     OpenAiEmbedder,
+    RestEmbedder,
+    UserProvidedEmbedder,
+)
+from meilisearch.models.index import (
+    Faceting,
+    IndexStats,
+    LocalizedAttributes,
     Pagination,
+    PrefixSearch,
     ProximityPrecision,
     TypoTolerance,
-    UserProvidedEmbedder,
 )
 from meilisearch.models.task import Task, TaskInfo, TaskResults
 from meilisearch.task import TaskHandler
@@ -268,25 +276,27 @@ class Index:
             An error containing details about why Meilisearch can't process your request. Meilisearch error codes are described here: https://www.meilisearch.com/docs/reference/errors/error_codes#meilisearch-errors
         """
         stats = self.http.get(f"{self.config.paths.index}/{self.uid}/{self.config.paths.stat}")
-        return IndexStats(stats)
+        return IndexStats(**stats)
 
     @version_error_hint_message
     def search(self, query: str, opt_params: Optional[Mapping[str, Any]] = None) -> Dict[str, Any]:
         """Search in the index.
 
+        https://www.meilisearch.com/docs/reference/api/search
+
         Parameters
         ----------
         query:
             String containing the searched word(s)
         opt_params (optional):
             Dictionary containing optional query parameters.
-            Note: The vector parameter is only available in Meilisearch >= v1.3.0, and is experimental
-            Meilisearch v1.3.0. In order to use this feature in Meilisearch v1.3.0 you first need to
-            enable the feature by sending a PATCH request to /experimental-features with
-            { "vectoreStore": true }. Because this feature is experimental it may be removed or
-            updated causing breaking changes in this library without a major version bump so use
-            with caution.
-            https://www.meilisearch.com/docs/reference/api/search#search-in-an-index
+            Common parameters include:
+            - hybrid: Dict with 'semanticRatio' and 'embedder' fields for hybrid search
+            - vector: Array of numbers for vector search
+            - retrieveVectors: Boolean to include vector data in search results
+            - filter: Filter queries by an attribute's value
+            - limit: Maximum number of documents returned
+            - offset: Number of documents to skip
 
         Returns
         -------
@@ -300,7 +310,9 @@ class Index:
         """
         if opt_params is None:
             opt_params = {}
+
         body = {"q": query, **opt_params}
+
         return self.http.post(
             f"{self.config.paths.index}/{self.uid}/{self.config.paths.search}",
             body=body,
@@ -363,7 +375,7 @@ class Index:
         """
         if parameters is None:
             parameters = {}
-        elif "fields" in parameters and isinstance(parameters["fields"], list):
+        elif "fields" in parameters and isinstance(parameters["fields"], (list, tuple)):
             parameters["fields"] = ",".join(parameters["fields"])
 
         document = self.http.get(
@@ -397,23 +409,38 @@ class Index:
         MeilisearchApiError
             An error containing details about why Meilisearch can't process your request. Meilisearch error codes are described here: https://www.meilisearch.com/docs/reference/errors/error_codes#meilisearch-errors
         """
-        if parameters is None or parameters.get("filter") is None:
-            if parameters is None:
-                parameters = {}
-            elif "fields" in parameters and isinstance(parameters["fields"], list):
-                parameters["fields"] = ",".join(parameters["fields"])
-
-            response = self.http.get(
-                f"{self.config.paths.index}/{self.uid}/{self.config.paths.document}?{parse.urlencode(parameters)}"
-            )
-            return DocumentsResults(response)
-
+        if parameters is None:
+            parameters = {}
         response = self.http.post(
             f"{self.config.paths.index}/{self.uid}/{self.config.paths.document}/fetch",
             body=parameters,
         )
         return DocumentsResults(response)
 
+    def get_similar_documents(self, parameters: Mapping[str, Any]) -> Dict[str, Any]:
+        """Get the documents similar to a document.
+
+        Parameters
+        ----------
+        parameters:
+            parameters accepted by the get similar documents route: https://www.meilisearch.com/docs/reference/api/similar#body
+            "id" and "embedder" are required.
+
+        Returns
+        -------
+        results:
+            Dictionary with hits, offset, limit, processingTimeMs, and id
+
+        Raises
+        ------
+        MeilisearchApiError
+            An error containing details about why Meilisearch can't process your request. Meilisearch error codes are described here: https://www.meilisearch.com/docs/reference/errors/error_codes#meilisearch-errors
+        """
+        return self.http.post(
+            f"{self.config.paths.index}/{self.uid}/{self.config.paths.similar}",
+            body=parameters,
+        )
+
     def add_documents(
         self,
         documents: Sequence[Mapping[str, Any]],
@@ -493,17 +520,17 @@ class Index:
 
     def add_documents_json(
         self,
-        str_documents: str,
+        str_documents: bytes,
         primary_key: Optional[str] = None,
         *,
         serializer: Optional[Type[JSONEncoder]] = None,
     ) -> TaskInfo:
-        """Add string documents from JSON file to the index.
+        """Add documents to the index from a byte-encoded JSON string.
 
         Parameters
         ----------
         str_documents:
-            String of document from a JSON file.
+            Byte-encoded JSON string.
         primary_key (optional):
             The primary-key used in index. Ignored if already set up.
         serializer (optional):
@@ -527,16 +554,16 @@ class Index:
 
     def add_documents_csv(
         self,
-        str_documents: str,
+        str_documents: bytes,
         primary_key: Optional[str] = None,
         csv_delimiter: Optional[str] = None,
     ) -> TaskInfo:
-        """Add string documents from a CSV file to the index.
+        """Add documents to the index from a byte-encoded CSV string.
 
         Parameters
         ----------
         str_documents:
-            String of document from a CSV file.
+            Byte-encoded CSV string.
         primary_key (optional):
             The primary-key used in index. Ignored if already set up.
         csv_delimiter:
@@ -557,15 +584,15 @@ class Index:
 
     def add_documents_ndjson(
         self,
-        str_documents: str,
+        str_documents: bytes,
         primary_key: Optional[str] = None,
     ) -> TaskInfo:
-        """Add string documents from a NDJSON file to the index.
+        """Add documents to the index from a byte-encoded NDJSON string.
 
         Parameters
         ----------
         str_documents:
-            String of document from a NDJSON file.
+            Byte-encoded NDJSON string.
         primary_key (optional):
             The primary-key used in index. Ignored if already set up.
 
@@ -584,24 +611,24 @@ class Index:
 
     def add_documents_raw(
         self,
-        str_documents: str,
+        str_documents: bytes,
         primary_key: Optional[str] = None,
         content_type: Optional[str] = None,
         csv_delimiter: Optional[str] = None,
         *,
         serializer: Optional[Type[JSONEncoder]] = None,
     ) -> TaskInfo:
-        """Add string documents to the index.
+        """Add documents to the index from a byte-encoded string.
 
         Parameters
         ----------
         str_documents:
-            String of document.
+            Byte-encoded string.
+        content_type:
+            The content MIME type: 'application/json', 'application/x-dnjson', or 'text/csv'.
         primary_key (optional):
             The primary-key used in index. Ignored if already set up.
-        type:
-            The type of document. Type available: 'csv', 'json', 'jsonl'.
-        csv_delimiter:
+        csv_delimiter (optional):
             One ASCII character used to customize the delimiter for CSV.
             Note: The csv delimiter can only be used with the Content-Type text/csv.
         serializer (optional):
@@ -942,12 +969,18 @@ class Index:
         )
 
         if settings.get("embedders"):
-            embedders: dict[str, OpenAiEmbedder | HuggingFaceEmbedder | UserProvidedEmbedder] = {}
+            embedders: dict[str, EmbedderType] = {}
             for k, v in settings["embedders"].items():
                 if v.get("source") == "openAi":
                     embedders[k] = OpenAiEmbedder(**v)
+                elif v.get("source") == "ollama":
+                    embedders[k] = OllamaEmbedder(**v)
                 elif v.get("source") == "huggingFace":
                     embedders[k] = HuggingFaceEmbedder(**v)
+                elif v.get("source") == "rest":
+                    embedders[k] = RestEmbedder(**v)
+                elif v.get("source") == "composite":
+                    embedders[k] = CompositeEmbedder(**v)
                 else:
                     embedders[k] = UserProvidedEmbedder(**v)
 
@@ -955,7 +988,7 @@ class Index:
 
         return settings
 
-    def update_settings(self, body: Mapping[str, Any]) -> TaskInfo:
+    def update_settings(self, body: MutableMapping[str, Any]) -> TaskInfo:
         """Update settings of the index.
 
         https://www.meilisearch.com/docs/reference/api/settings#update-settings
@@ -964,6 +997,26 @@ class Index:
         ----------
         body:
             Dictionary containing the settings of the index.
+            Supported settings include:
+            - 'rankingRules': List of ranking rules
+            - 'distinctAttribute': Attribute for deduplication
+            - 'searchableAttributes': Attributes that can be searched
+            - 'displayedAttributes': Attributes to display in search results
+            - 'stopWords': Words ignored in search queries
+            - 'synonyms': Dictionary of synonyms
+            - 'filterableAttributes': Attributes that can be used for filtering
+            - 'sortableAttributes': Attributes that can be used for sorting
+            - 'typoTolerance': Settings for typo tolerance
+            - 'pagination': Settings for pagination
+            - 'faceting': Settings for faceting
+            - 'dictionary': List of custom dictionary words
+            - 'separatorTokens': List of separator tokens
+            - 'nonSeparatorTokens': List of non-separator tokens
+            - 'embedders': Dictionary of embedder configurations for AI-powered search
+            - 'searchCutoffMs': Maximum search time in milliseconds
+            - 'proximityPrecision': Precision for proximity ranking
+            - 'localizedAttributes': Settings for localized attributes
+
             More information:
             https://www.meilisearch.com/docs/reference/api/settings#update-settings
 
@@ -976,8 +1029,14 @@ class Index:
         Raises
         ------
         MeilisearchApiError
-            An error containing details about why Meilisearch can't process your request. Meilisearch error codes are described here: https://www.meilisearch.com/docs/reference/errors/error_codes#meilisearch-errors
+            An error containing details about why Meilisearch can't process your request.
+            Meilisearch error codes are described here: https://www.meilisearch.com/docs/reference/errors/error_codes#meilisearch-errors
         """
+        if body.get("embedders"):
+            for _, v in body["embedders"].items():
+                if "documentTemplateMaxBytes" in v and v["documentTemplateMaxBytes"] is None:
+                    del v["documentTemplateMaxBytes"]
+
         task = self.http.patch(
             f"{self.config.paths.index}/{self.uid}/{self.config.paths.setting}", body
         )
@@ -1607,6 +1666,57 @@ class Index:
 
         return TaskInfo(**task)
 
+    def get_facet_search_settings(self) -> bool:
+        """Get the facet search settings of an index.
+
+        Returns
+        -------
+        bool:
+            True if facet search is enabled, False if disabled.
+        Raises
+        ------
+        MeilisearchApiError
+            An error containing details about why Meilisearch can't process your request. Meilisearch error codes are described here: https://www.meilisearch.com/docs/reference/errors/error_codes#meilisearch-errors
+        """
+
+        return self.http.get(self.__settings_url_for(self.config.paths.facet_search))
+
+    def update_facet_search_settings(self, body: Union[bool, None]) -> TaskInfo:
+        """Update the facet search settings of the index.
+
+        Parameters
+        ----------
+        body: bool
+            True to enable facet search, False to disable it.
+
+        Returns
+        -------
+        task_info:
+            TaskInfo instance containing information about a task to track the progress of an asynchronous process.
+            https://www.meilisearch.com/docs/reference/api/tasks#get-one-task
+
+        Raises
+        ------
+        MeilisearchApiError
+            An error containing details about why Meilisearch can't process your request. Meilisearch error codes are described here: https://www.meilisearch.com/docs/reference/errors/error_codes#meilisearch-errors
+        """
+        task = self.http.put(self.__settings_url_for(self.config.paths.facet_search), body=body)
+
+        return TaskInfo(**task)
+
+    def reset_facet_search_settings(self) -> TaskInfo:
+        """Reset facet search settings of the index to default values.
+
+        Returns
+        -------
+        task_info:
+            TaskInfo instance containing information about a task to track the progress of an asynchronous process.
+            https://www.meilisearch.com/docs/reference/api/tasks
+        """
+        task = self.http.delete(self.__settings_url_for(self.config.paths.facet_search))
+
+        return TaskInfo(**task)
+
     def get_faceting_settings(self) -> Faceting:
         """Get the faceting settings of an index.
 
@@ -1620,7 +1730,6 @@ class Index:
         MeilisearchApiError
             An error containing details about why Meilisearch can't process your request. Meilisearch error codes are described here: https://www.meilisearch.com/docs/reference/errors/error_codes#meilisearch-errors
         """
-
         faceting = self.http.get(self.__settings_url_for(self.config.paths.faceting))
 
         return Faceting(**faceting)
@@ -1850,10 +1959,13 @@ class Index:
     def get_embedders(self) -> Embedders | None:
         """Get embedders of the index.
 
+        Retrieves the current embedder configuration from Meilisearch.
+
         Returns
         -------
-        settings:
-            The embedders settings of the index.
+        Embedders:
+            The embedders settings of the index, or None if no embedders are configured.
+            Contains a dictionary of embedder configurations, where keys are embedder names.
 
         Raises
         ------
@@ -1865,25 +1977,37 @@ class Index:
         if not response:
             return None
 
-        embedders: dict[str, OpenAiEmbedder | HuggingFaceEmbedder | UserProvidedEmbedder] = {}
+        embedders: dict[str, EmbedderType] = {}
         for k, v in response.items():
-            print(v.get("source"))
-            if v.get("source") == "openAi":
+            source = v.get("source")
+            if source == "openAi":
                 embedders[k] = OpenAiEmbedder(**v)
-            elif v.get("source") == "huggingFace":
+            elif source == "huggingFace":
                 embedders[k] = HuggingFaceEmbedder(**v)
+            elif source == "ollama":
+                embedders[k] = OllamaEmbedder(**v)
+            elif source == "rest":
+                embedders[k] = RestEmbedder(**v)
+            elif source == "composite":
+                embedders[k] = CompositeEmbedder(**v)
+            elif source == "userProvided":
+                embedders[k] = UserProvidedEmbedder(**v)
             else:
+                # Default to UserProvidedEmbedder for unknown sources
                 embedders[k] = UserProvidedEmbedder(**v)
 
         return Embedders(embedders=embedders)
 
-    def update_embedders(self, body: Union[Mapping[str, Any], None]) -> TaskInfo:
+    def update_embedders(self, body: Union[MutableMapping[str, Any], None]) -> TaskInfo:
         """Update embedders of the index.
 
+        Updates the embedder configuration for the index. The embedder configuration
+        determines how Meilisearch generates vector embeddings for documents.
+
         Parameters
         ----------
         body: dict
-            Dictionary containing the embedders.
+            Dictionary containing the embedders configuration.
 
         Returns
         -------
@@ -1894,8 +2018,31 @@ class Index:
         Raises
         ------
         MeilisearchApiError
-            An error containing details about why Meilisearch can't process your request. Meilisearch error codes are described here: https://www.meilisearch.com/docs/reference/errors/error_codes#meilisearch-errors
+            An error containing details about why Meilisearch can't process your request.
+            Meilisearch error codes are described here: https://www.meilisearch.com/docs/reference/errors/error_codes#meilisearch-errors
         """
+        if body is not None and body.get("embedders"):
+            embedders: dict[str, EmbedderType] = {}
+            for k, v in body["embedders"].items():
+                source = v.get("source")
+                if source == "openAi":
+                    embedders[k] = OpenAiEmbedder(**v)
+                elif source == "huggingFace":
+                    embedders[k] = HuggingFaceEmbedder(**v)
+                elif source == "ollama":
+                    embedders[k] = OllamaEmbedder(**v)
+                elif source == "rest":
+                    embedders[k] = RestEmbedder(**v)
+                elif source == "composite":
+                    embedders[k] = CompositeEmbedder(**v)
+                elif source == "userProvided":
+                    embedders[k] = UserProvidedEmbedder(**v)
+                else:
+                    # Default to UserProvidedEmbedder for unknown sources
+                    embedders[k] = UserProvidedEmbedder(**v)
+
+            body = {"embedders": {k: v.model_dump(by_alias=True) for k, v in embedders.items()}}
+
         task = self.http.patch(self.__settings_url_for(self.config.paths.embedders), body)
 
         return TaskInfo(**task)
@@ -1903,6 +2050,8 @@ class Index:
     def reset_embedders(self) -> TaskInfo:
         """Reset embedders of the index to default values.
 
+        Removes all embedder configurations from the index.
+
         Returns
         -------
         task_info:
@@ -1980,6 +2129,58 @@ class Index:
 
         return TaskInfo(**task)
 
+    # PREFIX SEARCH
+
+    def get_prefix_search(self) -> PrefixSearch:
+        """Get the prefix search settings of an index.
+
+        Returns
+        -------
+        settings:
+            The prefix search settings of the index.
+
+        Raises
+        ------
+        MeilisearchApiError
+            An error containing details about why Meilisearch can't process your request. Meilisearch error codes are described here: https://www.meilisearch.com/docs/reference/errors/error_codes#meilisearch-errors
+        """
+        prefix_search = self.http.get(self.__settings_url_for(self.config.paths.prefix_search))
+
+        return PrefixSearch[to_snake(prefix_search).upper()]
+
+    def update_prefix_search(self, body: Union[PrefixSearch, None]) -> TaskInfo:
+        """Update the prefix search settings of the index.
+
+        Parameters
+        ----------
+        body:
+            Prefix search settings
+
+        Returns
+        -------
+        task_info:
+            TaskInfo instance containing information about a task to track the progress of an asynchronous process.
+            https://www.meilisearch.com/docs/reference/api/tasks
+        """
+        task = self.http.put(self.__settings_url_for(self.config.paths.prefix_search), body)
+
+        return TaskInfo(**task)
+
+    def reset_prefix_search(self) -> TaskInfo:
+        """Reset the prefix search settings of the index
+
+        Returns
+        -------
+        task_info:
+            TaskInfo instance containing information about a task to track the progress of an asynchronous process.
+            https://www.meilisearch.com/docs/reference/api/tasks
+        """
+        task = self.http.delete(
+            self.__settings_url_for(self.config.paths.prefix_search),
+        )
+
+        return TaskInfo(**task)
+
     # PROXIMITY PRECISION SETTINGS
 
     def get_proximity_precision(self) -> ProximityPrecision:
@@ -2041,6 +2242,73 @@ class Index:
 
         return TaskInfo(**task)
 
+    # LOCALIZED ATTRIBUTES SETTINGS
+
+    def get_localized_attributes(self) -> Union[List[LocalizedAttributes], None]:
+        """Get the localized_attributes of the index.
+
+        Returns
+        -------
+        settings:
+            localized_attributes of the index.
+
+        Raises
+        ------
+        MeilisearchApiError
+            An error containing details about why Meilisearch can't process your request. Meilisearch error codes are described here: https://www.meilisearch.com/docs/reference/errors/error_codes#meilisearch-errors
+        """
+        response = self.http.get(self.__settings_url_for(self.config.paths.localized_attributes))
+
+        if not response:
+            return None
+
+        return [LocalizedAttributes(**attrs) for attrs in response]
+
+    def update_localized_attributes(
+        self, body: Union[List[Mapping[str, List[str]]], None]
+    ) -> TaskInfo:
+        """Update the localized_attributes of the index.
+
+        Parameters
+        ----------
+        body:
+            localized_attributes
+
+        Returns
+        -------
+        task_info:
+            TaskInfo instance containing information about a task to track the progress of an asynchronous process.
+            https://www.meilisearch.com/docs/reference/api/tasks#get-one-task
+
+        Raises
+        ------
+        MeilisearchApiError
+            An error containing details about why Meilisearch can't process your request. Meilisearch error codes are described here: https://www.meilisearch.com/docs/reference/errors/error_codes#meilisearch-errors
+        """
+        task = self.http.put(self.__settings_url_for(self.config.paths.localized_attributes), body)
+
+        return TaskInfo(**task)
+
+    def reset_localized_attributes(self) -> TaskInfo:
+        """Reset the localized_attributes of the index
+
+        Returns
+        -------
+        task_info:
+            TaskInfo instance containing information about a task to track the progress of an asynchronous process.
+            https://www.meilisearch.com/docs/reference/api/tasks#get-one-task
+
+        Raises
+        ------
+        MeilisearchApiError
+            An error containing details about why Meilisearch can't process your request. Meilisearch error codes are described here: https://www.meilisearch.com/docs/reference/errors/error_codes#meilisearch-errors
+        """
+        task = self.http.delete(
+            self.__settings_url_for(self.config.paths.localized_attributes),
+        )
+
+        return TaskInfo(**task)
+
     @staticmethod
     def _batch(
         documents: Sequence[Mapping[str, Any]], batch_size: int
